/************************************************************************************
CommandSelector RT-Component
Copyright (c) 2010, Kyoto Uni.
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD lisence.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/

// -*- C++ -*-
/*!
 * @file  CommandSelector.cpp
 * @brief velocity selector
 * @date $Date$
 *
 * $Id$
 */

#include "CommandSelector.h"

// Module specification
// <rtc-template block="module_spec">
static const char* commandselector_spec[] =
  {
    "implementation_id", "CommandSelector",
    "type_name",         "CommandSelector",
    "description",       "velocity selector",
    "version",           "1.0.0",
    "vendor",            "matsunolab  Kyoto-u",
    "category",          "Generic",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    "conf.default.timeout","1000",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
CommandSelector::CommandSelector(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_velocity_manualIn("velocity_manual", m_velocity_manual),
    m_velocity_autoIn("velocity_auto", m_velocity_auto),
    m_select_autoIn("select_auto", m_select_auto),
    m_emo_stateIn("emo_state", m_emo_state),
    m_velocity_selectedOut("velocity_selected", m_velocity_selected),
    m_modeOut("mode", m_mode)
    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
CommandSelector::~CommandSelector()
{
}



RTC::ReturnCode_t CommandSelector::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("select_auto", m_select_autoIn);
  addInPort("velocity_manual", m_velocity_manualIn);
  addInPort("velocity_auto", m_velocity_autoIn);
  addInPort("emo_state", m_emo_stateIn);
  
  // Set OutPort buffer
  addOutPort("velocity_selected", m_velocity_selectedOut);
  addOutPort("mode", m_modeOut);

  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports

  bindParameter("timeout",m_timeout,"1000");
  m_configsets.update("default");

  // </rtc-template>

  printf("-----------------------------\n");
  printf("CommandSelector Module\n");
  printf("-----------------------------\n");
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t CommandSelector::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CommandSelector::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CommandSelector::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t CommandSelector::onActivated(RTC::UniqueId ec_id)
{
  is_auto = false;
  is_emo  = true;
  
  for(int i= 0;i<4;i++)
  {
	  timer[i].Clear();
      timer[i].SetTimeout( (double)m_timeout/1000 );
  }

  m_velocity_manual.data.vx = 0.0;
  m_velocity_manual.data.vy = 0.0;
  m_velocity_manual.data.va = 0.0;

  m_velocity_auto.data.vx = 0.0;
  m_velocity_auto.data.vy = 0.0;
  m_velocity_auto.data.va = 0.0;

  m_velocity_selected.data.vx = 0.0;
  m_velocity_selected.data.vy = 0.0;
  m_velocity_selected.data.va = 0.0;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t CommandSelector::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t CommandSelector::onExecute(RTC::UniqueId ec_id)
{
  //Read data from inports  
  if( m_velocity_manualIn.isNew() )
  {
	m_velocity_manualIn.read();
	timer[0].SetCurrentTime();
  }
  if( m_velocity_autoIn.isNew() )
  {
	m_velocity_autoIn.read();
    timer[1].SetCurrentTime();
  }

  if( m_select_autoIn.isNew() )
  {
   	m_select_autoIn.read();
	is_auto = m_select_auto.data;
	timer[2].SetCurrentTime();
  }
  if( m_emo_stateIn.isNew() )
  {
	m_emo_stateIn.read();
	is_emo = m_emo_state.data;
	timer[3].SetCurrentTime();
  }

  //At least m_emoIn has to be connected 
  if( !timer[3].isConnect() )
  {
	 is_auto = false;
	 is_emo  = true;
  }
  //if isnot connected, is_auto is turned to false(i.e. manual) 
  //if( !timer[2].isConnect() )
  //{ 
  //	is_auto = false;
  //}
  //Select velocity
  if( is_emo  )//Emergency
  {
	  m_velocity_selected.data.vx = 0.0;
	  m_velocity_selected.data.vy = 0.0;
	  m_velocity_selected.data.va = 0.0;
	  m_mode.data = 1;
	  is_auto = false;
      printf("EMO Status       : Emergency                                              \r");
  }
  else
  {
	  if( !is_auto && timer[0].isConnect()) //Manual
	  {
		m_velocity_selected = m_velocity_manual;
		printf("Velocity : Manual  ,vx:%.3lf[m/s] vy:%.3lf[m/s] va:%.3lf[rad/s]\r",
			m_velocity_selected.data.vx,m_velocity_selected.data.vy,m_velocity_selected.data.va);
		m_mode.data = 2;
	  }
	  else if( timer[1].isConnect() ) //Autonomous
	  {
		m_velocity_selected = m_velocity_auto;
		printf("Velocity : Auto    ,vx:%.3lf[m/s] vy:%.3lf[m/s] va:%.3lf[rad/s]\r",
			m_velocity_selected.data.vx,m_velocity_selected.data.vy,m_velocity_selected.data.va);	  
	     m_mode.data =3;
	  }
  
	  else
      {
	    m_velocity_selected.data.vx = 0.0;
 		m_velocity_selected.data.vy = 0.0;
 		m_velocity_selected.data.va = 0.0;
		m_mode.data = 0;
      }
  }
  m_velocity_selectedOut.write();  
  m_modeOut.write();

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t CommandSelector::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CommandSelector::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CommandSelector::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CommandSelector::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CommandSelector::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void CommandSelectorInit(RTC::Manager* manager)
  {
    coil::Properties profile(commandselector_spec);
    manager->registerFactory(profile,
                             RTC::Create<CommandSelector>,
                             RTC::Delete<CommandSelector>);
  }
  
};


